/*
 *  Copyright (c) 1999-2002 ChurchillObjects.com  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are
 *  met: Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer. Redistributions in
 *  binary form must reproduce the above copyright notice, this list of
 *  conditions and the following disclaimer in the documentation and/or other
 *  materials provided with the distribution. Neither the name of the copyright
 *  holder nor the names of its contributors may be used to endorse or promote
 *  products derived from this software without specific prior written
 *  permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
 *  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT, INCLUDING NEGLIGENCE OR OTHERWISE, ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 *  DAMAGE.
 *
 */

package churchillobjects.rss4j;

import java.io.Serializable;
import java.util.Vector;
import java.util.Enumeration;
import churchillobjects.rss4j.model.IUsesDublinCore;
import churchillobjects.rss4j.model.IUsesSyndication;

/**
 * Encapsulates the data and some minor logic regarding a channel in the RSS
 * object model. The channel contains all of the elements including items, images, etc.
 * that pertain to a single channel. There may be more than one channel in a
 * document. A channel may also have one syndication.
 */
public class RssChannel implements IUsesSyndication, IUsesDublinCore, Cloneable, Serializable {

	/**
	 * Permissible skipdays text items. Any skipdays settings is checked against
	 * this list for accuracy (i.e. spelling). Only used for RSS 0.91.
	 */
	public static final String[] SKIPDAYS = {
		"Monday", "Tuesday", "Wednesday", "Thursday",
		"Friday", "Saturday", "Sunday"};

	/**
	* The channel title.
	*/
	private String channelTitle;

	/**
	 * The channel description.
	 */
	private String channelDescription;

	/**
	 * The channel link.
	 */
	private String channelLink;

	/**
	 * The channel uri (where it came from).
	 */
	private String channelUri;

	/**
	 * The channel language.
	 */
	private String channelLanguage;

	/**
	 * The channel copyright.
	 */
	private String channelCopyright;

	/**
	 * The channel pub date.
	 */
	private String channelPubDate;

	/**
	 * The channel's last build date.
	 */
	private String channelLastBuildDate;

	/**
	 * The channel's managing editor.
	 */
	private String channelManagingEditor;

	/**
	 * The channel's webmaster.
	 */
	private String channelWebmaster;

	/**
	 * The channel's PICS rating.
	 */
	private String channelPicsRating;

	/**
	 * The channels docs.
	 */
	private String channelDocs;

	/**
	 * The channel's syndication information.
	 */
	private RssSyndication channelSyndication;

	/**
	 * The channel's dublin core information.
	 */
	private RssDublinCore dublinCore;

	/**
	 * A list of the channel's skipdays.
	 */
	private Vector skipDays = new Vector();

	/**
	 * A list of the channel's skiphours.
	 */
	private Vector skipHours = new Vector();

	/**
	 * The channel's image.
	 */
	private RssChannelImage channelImage;

	/**
	 * The channel's text input.
	 */
	private RssChannelTextInput channelTextInput;

	/**
	 * A list of the channel's items.
	 */
	private Vector items = new Vector();

	/**
	 * Sets the channel docs (RSS 0.91 only).
	 * @param s
	 */
	public final void setChannelDocs(String s){
		channelDocs = s;
	}

	/**
	 * Returns the channel docs (RSS 0.91 only).
	 * @return
	 */
	public final String getChannelDocs(){
		return channelDocs;
	}

	/**
	 * Sets the channel's text input object.
	 * @param input
	 */
	public final void setChannelTextInput(RssChannelTextInput input){
		channelTextInput = input;
	}

	/**
	 * Returns the channel's text input object.
	 * @return
	 */
	public final RssChannelTextInput getChannelTextInput(){
		return channelTextInput;
	}

	/**
	 * Sets the channel image to the specified object.
	 * @param image
	 */
	public final void setChannelImage(RssChannelImage image){
		this.channelImage = image;
	}

	/**
	 * Returns the encapsulated channel image object.
	 * @return
	 */
	public final RssChannelImage getChannelImage(){
		return channelImage;
	}

	/**
	 * Returns the channel title.
	 * @return
	 */
	public final String getChannelTitle(){
		return channelTitle;
	}

	/**
	 * Sets the channel title.
	 * @param ct
	 */
	public final void setChannelTitle(String ct){
		channelTitle = ct;
	}

	/**
	 * Returns the channel description.
	 * @return
	 */
	public final String getChannelDescription(){
		return channelDescription;
	}

	/**
	 * Sets the channel description.
	 * @param cd
	 */
	public final void setChannelDescription(String cd){
		channelDescription = cd;
	}

	/**
	 * Returns the channel link.
	 * @return
	 */
	public final String getChannelLink(){
		return channelLink;
	}

	/**
	 * Sets the channel link.
	 * @param cl
	 */
	public final void setChannelLink(String cl){
		channelLink = cl;
	}

	/**
	 * Returns the channel language.
	 * @return
	 */
	public final String getChannelLanguage(){
		return channelLanguage;
	}

	/**
	 * Sets the channel language.
	 * @param cl
	 */
	public final void setChannelLanguage(String cl){
		channelLanguage = cl;
	}

	/**
	 * Returns the channel copyright.
	 * @return
	 */
	public final String getChannelCopyright(){
		return channelCopyright;
	}

	/**
	 * Sets the channel copyright.
	 * @param cr
	 */
	public final void setChannelCopyright(String cr){
		channelCopyright= cr;
	}

	/**
	 * Returns the channel publication date.
	 * @return
	 */
	public final String getChannelPubDate(){
		return channelPubDate;
	}

	/**
	 * Sets the channel publication date.
	 * @param pd
	 */
	public final void setChannelPubDate(String pd){
		channelPubDate = pd;
	}

	/**
	 * Returns the channel last build date.
	 * @return
	 */
	public final String getChannelLastBuildDate(){
		return channelLastBuildDate;
	}

	/**
	 * Sets the channel last build date.
	 * @param lbd
	 */
	public final void setChannelLastBuildDate(String lbd){
		channelLastBuildDate = lbd;
	}

	/**
	 * Returns the channel managing editor.
	 * @return
	 */
	public final String getChannelManagingEditor(){
		return channelManagingEditor;
	}

	/**
	 * Sets the channel managing editor.
	 * @param me
	 */
	public final void setChannelManagingEditor(String me){
		channelManagingEditor = me;
	}

	/**
	 * Returns the channel webmaster.
	 * @return
	 */
	public final String getChannelWebmaster(){
		return channelWebmaster;
	}

	/**
	 * Sets the channel webmaster.
	 * @param s
	 */
	public final void setChannelWebmaster(String s){
		channelWebmaster = s;
	}

	/**
	 * Sets the channel PICS rating.
	 * @param r
	 */
	public final void setChannelPicsRating(String r){
		channelPicsRating = r;
	}

	/**
	 * Returns the channel PICS rating.
	 * @return
	 */
	public final String getChannelPicsRating(){
		return channelPicsRating;
	}

	/**
	 * Adds the specified item object to the list of items for the channel.
	 * @param item
	 */
	public final void addItem(RssChannelItem item){
		items.addElement(item);
	}

	/**
	 * Returns the channel URI.
	 * @return
	 */
	public final String getChannelUri(){
		return channelUri;
	}

	/**
	 * Sets the channel URI.
	 * @param uri
	 */
	public final void setChannelUri(String uri){
		channelUri = uri;
	}

	/**
	 * Returns an enumeration of the channel's items.
	 * @return
	 */
	public final Enumeration items(){
		return items.elements();
	}

	/**
	 * Returns the size of the channel items list.
	 * @return
	 */
	public final int getItemCount(){
		return items.size();
	}

	/**
	 * Adds a skip day to the channel.
	 * @param s
	 */
	public final void addSkipDay(String s){
		for(int i=0;i<SKIPDAYS.length;i++){
			if(SKIPDAYS[i].equalsIgnoreCase(s)){
				if(!skipDays.contains(SKIPDAYS[i])){
					skipDays.addElement(SKIPDAYS[i]);
				}
				return;
			}
		}
		throw new IllegalArgumentException("Invalid value for skip days: '"+s+"'");
	}

	/**
	 * Returns an enumeration of the channel's skip days list.
	 * @return
	 */
	public final Enumeration skipDays(){
		return skipDays.elements();
	}

	/**
	 * Adds a skip hour to the channel's skip hours list.
	 * @param i
	 */
	public final void addSkipHour(int i){
		if(i<0 || i>23){
			throw new IllegalArgumentException("Invalid range for skip hours. Must be between 0 and 23: "+i);
		}
		Integer ii = new Integer(i);
		if(!skipHours.contains(ii)){
			skipHours.addElement(ii);
		}
	}

	/**
	 * Returns an enumeration of the channel's skip hours.
	 * @return
	 */
	public final Enumeration skipHours(){
		return skipHours.elements();
	}

	/**
	 * Sets the channel's syndication object.
	 * @param syn
	 */
	public final void setChannelSyndication(RssSyndication syn){
		channelSyndication = syn;
	}

	/**
	 * Returns the channel's syndication object.
	 * @return
	 */
	public final RssSyndication getChannelSyndication(){
		return channelSyndication;
	}

	/**
	 * Sets the channel's dublin core object.
	 * @param dc
	 */
	public final void setDublinCore(RssDublinCore dc){
		dublinCore = dc;
	}

	/**
	 * Returns the channel's dublin core object.
	 * @return
	 */
	public final RssDublinCore getDublinCore(){
		return dublinCore;
	}

	/**
	 * Returns a string representation of this object.
	 * @return
	 */
	public final String toString(){
		StringBuffer sb = new StringBuffer(300);
		sb.append("  RssChannel: [title='"+channelTitle+"' description='"+channelDescription+"'");
		if(channelLink!=null){
			sb.append(" link='"+channelLink+"'");
		}
		if(channelLanguage!=null){
			sb.append(" language='"+channelLanguage+"'");
		}
		if(channelCopyright!=null){
			sb.append(" copyright='"+channelCopyright+"'");
		}
		if(channelPubDate!=null){
			sb.append(" pubdate='"+channelPubDate+"'");
		}
		if(channelLastBuildDate!=null){
			sb.append(" last build date='"+channelLastBuildDate+"'");
		}
		if(channelManagingEditor!=null){
			sb.append(" managing editor='"+channelManagingEditor+"'");
		}
		if(channelWebmaster!=null){
			sb.append(" webmaster='"+channelWebmaster+"'");
		}
		if(channelPicsRating!=null){
			sb.append(" PICS rating='"+channelPicsRating+"'");
		}
		if(channelDocs!=null){
			sb.append(" docs='"+channelDocs+"'");
		}
		sb.append("]\n");
		if(skipDays!=null && skipDays.size()>0){
			sb.append("     skipDays: "+skipDays+"\n");
		}
		if(skipHours!=null && skipHours.size()>0){
			sb.append("     skipHours: "+skipHours+"\n");
		}
		if(channelImage!=null){
			sb.append("     image: "+channelImage+"\n");
		}
		if(channelTextInput!=null){
			sb.append("     textInput: "+channelTextInput+"\n");
		}
		if(dublinCore!=null){
			sb.append("     dublinCore: "+dublinCore+"\n");
		}
		if(channelSyndication!=null){
			sb.append("     syndication: "+channelSyndication+"\n");
		}
		Enumeration enumeration = items.elements();
		while(enumeration.hasMoreElements()){
			sb.append("     item: "+enumeration.nextElement()+"\n");
		}
		return sb.toString();
	}

	/**
	 * Computes the hashcode of this object based on the hashcodes of the
	 * objects it contains. For semantic reasons, an object with all null
	 * values returns 0, the same as if it were null within another object.
	 * @return The object's hashcode
	 */
	public final int hashCode(){
		int hc = (channelTitle==null ? 0 : channelTitle.hashCode());
		hc = 31 * hc + (channelDescription==null ? 0 : channelDescription.hashCode());
		hc = 31 * hc + (channelLink==null ? 0 : channelLink.hashCode());
		hc = 31 * hc + (channelLanguage==null ? 0 : channelLanguage.hashCode());
		hc = 31 * hc + (channelCopyright==null ? 0 : channelCopyright.hashCode());
		hc = 31 * hc + (channelPubDate==null ? 0 : channelPubDate.hashCode());
		hc = 31 * hc + (channelLastBuildDate==null ? 0 : channelLastBuildDate.hashCode());
		hc = 31 * hc + (channelManagingEditor==null ? 0 : channelManagingEditor.hashCode());
		hc = 31 * hc + (channelWebmaster==null ? 0 : channelWebmaster.hashCode());
		hc = 31 * hc + (channelPicsRating==null ? 0 : channelPicsRating.hashCode());
		hc = 31 * hc + (channelDocs==null ? 0 : channelDocs.hashCode());
		Enumeration e1 = skipDays.elements();
		while(e1.hasMoreElements()){
			Object obj = e1.nextElement();
			if(obj!=null){
				hc = 31 * hc + obj.hashCode();
			}
		}
		Enumeration e2 = skipHours.elements();
		while(e2.hasMoreElements()){
			Object obj = e2.nextElement();
			if(obj!=null){
				hc = 31 * hc + obj.hashCode();
			}
		}
		hc = 31 * hc + (channelImage==null ? 0 : channelImage.hashCode());
		hc = 31 * hc + (channelTextInput==null ? 0 : channelTextInput.hashCode());
		Enumeration e3 = items.elements();
		while(e3.hasMoreElements()){
			Object obj = e3.nextElement();
			if(obj!=null){
				hc = 31 * hc + obj.hashCode();
			}
		}
		hc = 31 * hc + (channelSyndication==null ? 0 : channelSyndication.hashCode());
		hc = 31 * hc + (dublinCore==null ? 0 : dublinCore.hashCode());
		return hc;
	}

	/**
	 * Canonical equals method. Two RssChannel objects will be equal if their
	 * hashcodes (and thus their contents) are equal.
	 * @param o The other object
	 * @return If the two objects are canonically equal
	 */
    public boolean equals( Object o )
    {
        if ( this == o )
        {
            return true;
        }
        if ( o == null || getClass() != o.getClass() )
        {
            return false;
        }

        final RssChannel channel = (RssChannel) o;

        if ( channelCopyright != null ? !channelCopyright.equals( channel.channelCopyright ) : channel.channelCopyright != null )
        {
            return false;
        }
        if ( channelDescription != null ? !channelDescription.equals( channel.channelDescription ) : channel.channelDescription != null )
        {
            return false;
        }
        if ( channelDocs != null ? !channelDocs.equals( channel.channelDocs ) : channel.channelDocs != null )
        {
            return false;
        }
        if ( channelImage != null ? !channelImage.equals( channel.channelImage ) : channel.channelImage != null )
        {
            return false;
        }
        if ( channelLanguage != null ? !channelLanguage.equals( channel.channelLanguage ) : channel.channelLanguage != null )
        {
            return false;
        }
        if ( channelLastBuildDate != null ? !channelLastBuildDate.equals( channel.channelLastBuildDate ) : channel.channelLastBuildDate != null )
        {
            return false;
        }
        if ( channelLink != null ? !channelLink.equals( channel.channelLink ) : channel.channelLink != null )
        {
            return false;
        }
        if ( channelManagingEditor != null ? !channelManagingEditor.equals( channel.channelManagingEditor ) : channel.channelManagingEditor != null )
        {
            return false;
        }
        if ( channelPicsRating != null ? !channelPicsRating.equals( channel.channelPicsRating ) : channel.channelPicsRating != null )
        {
            return false;
        }
        if ( channelPubDate != null ? !channelPubDate.equals( channel.channelPubDate ) : channel.channelPubDate != null )
        {
            return false;
        }
        if ( channelSyndication != null ? !channelSyndication.equals( channel.channelSyndication ) : channel.channelSyndication != null )
        {
            return false;
        }
        if ( channelTextInput != null ? !channelTextInput.equals( channel.channelTextInput ) : channel.channelTextInput != null )
        {
            return false;
        }
        if ( channelTitle != null ? !channelTitle.equals( channel.channelTitle ) : channel.channelTitle != null )
        {
            return false;
        }
        if ( channelUri != null ? !channelUri.equals( channel.channelUri ) : channel.channelUri != null )
        {
            return false;
        }
        if ( channelWebmaster != null ? !channelWebmaster.equals( channel.channelWebmaster ) : channel.channelWebmaster != null )
        {
            return false;
        }
        if ( dublinCore != null ? !dublinCore.equals( channel.dublinCore ) : channel.dublinCore != null )
        {
            return false;
        }
        if ( items != null ? !items.equals( channel.items ) : channel.items != null )
        {
            return false;
        }
        if ( skipDays != null ? !skipDays.equals( channel.skipDays ) : channel.skipDays != null )
        {
            return false;
        }
        if ( skipHours != null ? !skipHours.equals( channel.skipHours ) : channel.skipHours != null )
        {
            return false;
        }

        return true;
    }

    /**
	 * Public clone method to allow cloning.
	 * @return This object's clone
	 */
	public Object clone(){
		return this.clone();
	}

}