/*
 *  Copyright (c) 1999-2002 ChurchillObjects.com  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are
 *  met: Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer. Redistributions in
 *  binary form must reproduce the above copyright notice, this list of
 *  conditions and the following disclaimer in the documentation and/or other
 *  materials provided with the distribution. Neither the name of the copyright
 *  holder nor the names of its contributors may be used to endorse or promote
 *  products derived from this software without specific prior written
 *  permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
 *  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT, INCLUDING NEGLIGENCE OR OTHERWISE, ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 *  DAMAGE.
 *
 */

package churchillobjects.rss4j;

import java.io.Serializable;
import churchillobjects.rss4j.model.IUsesDublinCore;

/**
 * Encapsulates the data and some minor logic regarding an image in the RSS
 * object model. The image contains only those attributes pertaining to a single
 * image and an optional dublin core object.
 */
public class RssChannelImage implements IUsesDublinCore, Cloneable, Serializable{

	/**
	 * The image's title.
	 */
	private String imageTitle;

	/**
	 * The image's source url.
	 */
	private String imageUrl;

	/**
	 * The image's link url.
	 */
	private String imageLink;

	/**
	 * The image's width (zero is unset).
	 */
	private int imageWidth = 0;

	/**
	 * The image's height (zero is unset).
	 */
	private int imageHeight = 0;

	/**
	 * The image's dublin core.
	 */
	private RssDublinCore dublinCore;

	/**
	 * default constructor.
	 */
	public RssChannelImage(){
	}

	/**
	 * Overloaded constructor to set initial values for title, url and link.
	 * @param title
	 * @param url
	 * @param link
	 */
	public RssChannelImage(String title, String url, String link){
		imageTitle = title;
		imageUrl = url;
		imageLink = link;
	}

	/**
	 * Overloaded constructor to set initial values for title, url, link and size.
	 * @param title
	 * @param url
	 * @param link
	 * @param width
	 * @param height
	 */
	public RssChannelImage(String title, String url, String link, int width, int height){
		imageTitle = title;
		imageUrl = url;
		imageLink = link;
		imageWidth = width;
		imageHeight = height;
	}

	/**
	 * Sets the title of the image (can be used as ALT attribute)
	 * @param it
	 */
	public void setImageTitle(String t){
		imageTitle = t;
	}

	/**
	 * Returns the title of the image (can be used as ALT attribute)
	 * @return
	 */
	public String getImageTitle(){
		return imageTitle;
	}

	/**
	 * Sets the url of the image (the image source)
	 * @param s
	 */
	public void setImageUrl(String s){
		imageUrl = s;
	}

	/**
	 * Returns the url of the image (the image source)
	 * @return
	 */
	public String getImageUrl(){
		return imageUrl;
	}

	/**
	 * Sets the link of the image (where the browser should go if the user
	 * clicks on the image)
	 * @param link
	 */
	public void setImageLink(String l){
		imageLink = l;
	}

	/**
	 * Returns the link of the image (where the browser should go if the user
	 * clicks on the image)
	 * @return
	 */
	public String getImageLink(){
		return imageLink;
	}

	// Fields used by 0.91 only

	/**
	 * Sets the width of the image, in pixels. This value can be zero to "nullify"
	 * an existing value.
	 * @param i
	 */
	public void setImageWidth(int i){
		if(i<0){
			throw new IllegalArgumentException("Image width cannot be negative");
		}
		imageWidth = i;
	}

	/**
	 * Returns the width of the image, in pixels
	 * @return
	 */
	public int getImageWidth(){
		return imageWidth;
	}

	/**
	 * Sets the height of the image, in pixels. This value can be zero to "nullify"
	 * an existing value.
	 * @param i
	 */
	public void setImageHeight(int i){
		if(i<0){
			throw new IllegalArgumentException("Image height cannot be negative");
		}
		imageHeight = i;
	}

	/**
	 * Returns the height of the image, in pixels
	 * @return
	 */
	public int getImageHeight(){
		return imageHeight;
	}

	/**
	 * Sets the description for the channel image. For conversion, this value
	 * maps directly to the dublin core's description attribute.
	 * @param d
	 */
	public void setImageDescription(String d){
		if(dublinCore==null){
			dublinCore = new RssDublinCore();
		}
		dublinCore.setDescription(d);
	}

	/**
	 * Returns the description for the channel image. For conversion, this value
	 * maps directly to the dublin core's description attribute.
	 * @return
	 */
	public String getImageDescription(){
		if(dublinCore!=null){
			return dublinCore.getDescription();
		}
		return null;
	}

	// RSS 1.0 FIELDS

	/**
	 * Sets the dublin core for the channel image.
	 * @param dc
	 */
	public void setDublinCore(RssDublinCore dc){
		dublinCore = dc;
	}

	/**
	 * Returns the dublin core for the channel image.
	 * @return
	 */
	public RssDublinCore getDublinCore(){
		return dublinCore;
	}

	/**
	 * Returns a string representation of this object.
	 * @return
	 */
	public String toString(){
		StringBuffer sb = new StringBuffer();
		sb.append("[title='"+imageTitle+"', url='"+imageUrl+"', link='"+imageLink);
		if(imageWidth>0){
			sb.append(" width: "+imageWidth);
		}
		if(imageHeight>0){
			sb.append(" height: "+imageHeight);
		}
		if(dublinCore!=null){
			sb.append(" dublinCore: "+dublinCore);
		}
		sb.append("]");
		return sb.toString();
	}

	/**
	 * Computes the hashcode of this object based on the hashcodes of the
	 * objects it contains. For semantic reasons, an object with all null
	 * values returns 0, the same as if it were null within another object.
	 * @return The object's hashcode
	 */
	public int hashCode(){
		int hc = (imageTitle==null ? 0 : imageTitle.hashCode());
		hc = 31 * hc + (imageUrl==null ? 0 : imageUrl.hashCode());
		hc = 31 * hc + (imageLink==null ? 0 : imageLink.hashCode());
		hc = 31 * hc + imageWidth;
		hc = 31 * hc + imageHeight;
		hc = 31 * hc + (dublinCore==null ? 0 : dublinCore.hashCode());
		return hc;
	}

	/**
	 * Canonical equals method. Two RssChannel objects will be equal if their
	 * hashcodes (and thus their contents) are equal.
	 * @param obj The other object
	 * @return If the two objects are canonically equal
	 */
	public boolean equals(Object obj){
		if(obj==null || !(obj instanceof RssChannelImage)){
			return false;
		}
		if(this==obj){
			return true;
		}
		return this.hashCode()==obj.hashCode();
	}

	/**
	 * Public clone method to allow cloning.
	 * @return This object's clone
	 */
	public Object clone(){
		return this.clone();
	}

}