/*
 *  Copyright (c) 1999-2002 ChurchillObjects.com  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are
 *  met: Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer. Redistributions in
 *  binary form must reproduce the above copyright notice, this list of
 *  conditions and the following disclaimer in the documentation and/or other
 *  materials provided with the distribution. Neither the name of the copyright
 *  holder nor the names of its contributors may be used to endorse or promote
 *  products derived from this software without specific prior written
 *  permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
 *  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT, INCLUDING NEGLIGENCE OR OTHERWISE, ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 *  DAMAGE.
 *
 */

package churchillobjects.rss4j;

import java.io.Serializable;
import churchillobjects.rss4j.model.IUsesDublinCore;
import churchillobjects.rss4j.model.IUsesJbnPatch;

/**
 * Encapsulates the data and some minor logic regarding an item in the RSS
 * object model. The item contains only those attributes pertaining to a single
 * item and an optional dublin core object.
 */
public class RssChannelItem implements IUsesDublinCore, IUsesJbnPatch, Cloneable, Serializable{

	/**
	 * The title for the item.
	 */
	private String itemTitle;

	/**
	 * The link for the item (where the user goes when it is clicked on).
	 */
	private String itemLink;

	/**
	 * The description for the item.
	 */
	private String itemDescription;

	/**
	 * The dublin core for the item.
	 */
	private RssDublinCore dublinCore;

    /**
     * the JbnPatch info for the item
     * */
    private RssJbnPatch jbnPatch;

	/**
	 * Default constructor.
	 */
	public RssChannelItem(){
	}

	/**
	 * Overloaded constructor to set the title and link (required fields).
	 * @param title
	 * @param link
	 */
	public RssChannelItem(String title, String link){
		this.itemTitle = title;
		this.itemLink = link;
	}

	/**
	 * Overloaded constructor to set the title, link and description.
	 * @param title
	 * @param link
	 * @param description
	 */
	public RssChannelItem(String title, String link, String description){
		this.itemTitle = title;
		this.itemLink = link;
		this.itemDescription = description;
	}

	/**
	 * Sets the item title.
	 * @param t
	 */
	public void setItemTitle(String t){
		itemTitle = t;
	}

	/**
	 * Returns the item title.
	 * @return
	 */
	public String getItemTitle(){
		return itemTitle;
	}

	/**
	 * Sets the item link.
	 * @param l
	 */
	public void setItemLink(String l){
		itemLink = l;
	}

	/**
	 * Returns the item link.
	 * @return
	 */
	public String getItemLink(){
		return itemLink;
	}

	/**
	 * Sets the item description.
	 * @param d
	 */
	public void setItemDescription(String d){
		itemDescription = d;
	}

	/**
	 * Returns the item description.
	 * @return
	 */
	public String getItemDescription(){
		return itemDescription;
	}

	/**
	 * Sets the dublin core.
	 * @param dc
	 */
	public void setDublinCore(RssDublinCore dc){
		dublinCore = dc;
	}

	/**
	 * Returns the dublin core.
	 * @return
	 */
	public RssDublinCore getDublinCore(){
		return dublinCore;
	}

    /** sets the JbnPatch.
     * @return the JBossNetwork Patch
     */
    public RssJbnPatch getJbnPatch()
    {
        return jbnPatch;
    }

    /** returns the JbnPatch */
    public void setJbnPatch( RssJbnPatch jbnPatch )
    {
        this.jbnPatch = jbnPatch;
    }

	/**
	 * Returns a string representation of this object.
	 * @return
	 */
	public String toString(){
		return "[title='"+itemTitle+
				"', link='"+itemLink+
				", description='"+itemDescription+
				"']";
	}

	/**
	 * Computes the hashcode of this object based on the hashcodes of the
	 * objects it contains. For semantic reasons, an object with all null
	 * values returns 0, the same as if it were null within another object.
	 * @return The object's hashcode
	 */
	public int hashCode(){
		int hc = (itemTitle==null ? 0 : itemTitle.hashCode());
		hc = 31 * hc + (itemLink==null ? 0 : itemLink.hashCode());
		hc = 31 * hc + (itemDescription==null ? 0 : itemDescription.hashCode());
		hc = 31 * hc + (dublinCore==null ? 0 : dublinCore.hashCode());
        hc = 31 * hc + (jbnPatch== null ? 0: jbnPatch.hashCode());
        return hc;
	}

	/**
	 * Canonical equals method. Two RssChannel objects will be equal if their
	 * hashcodes (and thus their contents) are equal.
	 * @param o The other object
	 * @return If the two objects are canonically equal
	 */
	public boolean equals( Object o )
    {
        if ( this == o )
        {
            return true;
        }
        if ( o == null || getClass() != o.getClass() )
        {
            return false;
        }

        final RssChannelItem that = (RssChannelItem) o;

        if ( dublinCore != null ? !dublinCore.equals( that.dublinCore ) : that.dublinCore != null )
        {
            return false;
        }
        if ( itemDescription != null ? !itemDescription.equals( that.itemDescription ) : that.itemDescription != null )
        {
            return false;
        }
        if ( itemLink != null ? !itemLink.equals( that.itemLink ) : that.itemLink != null )
        {
            return false;
        }
        if ( itemTitle != null ? !itemTitle.equals( that.itemTitle ) : that.itemTitle != null )
        {
            return false;
        }
        if ( jbnPatch != null ? !jbnPatch.equals( that.jbnPatch ) : that.jbnPatch != null )
        {
            return false;
        }

        return true;
    }


    /**
	 * Public clone method to allow cloning.
	 * @return This object's clone
	 */
	public Object clone(){
		return this.clone();
	}
}
