/*
 *  Copyright (c) 1999-2002 ChurchillObjects.com  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are
 *  met: Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer. Redistributions in
 *  binary form must reproduce the above copyright notice, this list of
 *  conditions and the following disclaimer in the documentation and/or other
 *  materials provided with the distribution. Neither the name of the copyright
 *  holder nor the names of its contributors may be used to endorse or promote
 *  products derived from this software without specific prior written
 *  permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
 *  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT, INCLUDING NEGLIGENCE OR OTHERWISE, ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 *  DAMAGE.
 *
 */

package churchillobjects.rss4j;

import java.io.Serializable;
import churchillobjects.rss4j.model.IUsesDublinCore;
import churchillobjects.rss4j.model.IUsesJbnPatch;

/**
 * Encapsulates the data and some minor logic regarding an item in the RSS
 * object model. The item contains only those attributes pertaining to a single
 * item and an optional dublin core object.
 */
public class RssChannelTextInput implements IUsesDublinCore, IUsesJbnPatch, Cloneable, Serializable{

    /**
     * The title of the text input.
     */
    private String inputTitle;

    /**
     * A description for the text input.
     */
    private String inputDescription;

    /**
     * The name of the input (for the form).
     */
    private String inputName;

    /**
     * The link for the input (the form target).
     */
    private String inputLink;

    /**
     * The dublin core for the input.
     */
    private RssDublinCore dublinCore;

    /**
     * the jbnpatch for the input.
     */
    private RssJbnPatch jbnPatch;

    /**
     * Sets the input's title.
     * @param s
     */
    public void setInputTitle(String s){
        inputTitle = s;
    }

    /**
     * @return the input's title
     */
    public String getInputTitle(){
        return inputTitle;
    }

    /**
     * Sets the input's description.
     * @param s
     */
    public void setInputDescription(String s){
        inputDescription = s;
    }

    /**
     * @return the input's description.
     */
    public String getInputDescription(){
        return inputDescription;
    }

    /**
     * Sets the input's name (the name of the form's input tag, for the server
     * to parse when submitted).
     * @param s
     */
    public void setInputName(String s){
        inputName = s;
    }

    /**
     * @return the input's name (the name of the form's input tag, for the server
     * to parse when submitted)
     */
    public String getInputName(){
        return inputName;
    }

    /**
     * Sets the input's link (the action of the form).
     * @param s
     */
    public void setInputLink(String s){
        inputLink = s;
    }

    /**
     * @return the input's link (the action of the form).
     */
    public String getInputLink(){
        return inputLink;
    }

    /**
     * Sets the input's dublin core.
     * @param dc
     */
    public void setDublinCore(RssDublinCore dc){
        dublinCore = dc;
    }

    /**
     * @return the input's dublin core.
     */
    public RssDublinCore getDublinCore(){
        return dublinCore;
    }

    public void setJbnPatch( RssJbnPatch jbn )
    {
        jbnPatch = jbn;
    }

    public RssJbnPatch getJbnPatch()
    {
        return jbnPatch;
    }

    /**
     * Returns a string representation of this object.
     * @return
     */
    public String toString(){
        return "[title='"+inputTitle+
                "', description='"+inputDescription+
                "', inputName='"+inputName+
                "', inputLink='"+inputLink+
                "']";
    }

    /**
     * Computes the hashcode of this object based on the hashcodes of the
     * objects it contains. For semantic reasons, an object with all null
     * values returns 0, the same as if it were null within another object.
     * @return The object's hashcode
     */
    public int hashCode(){
        int hc = (inputTitle==null ? 0 : inputTitle.hashCode());
        hc = 31 * hc + (inputDescription==null ? 0 : inputDescription.hashCode());
        hc = 31 * hc + (inputName==null ? 0 : inputName.hashCode());
        hc = 31 * hc + (inputLink==null ? 0 : inputLink.hashCode());
        return hc;
    }

    /**
     * Canonical equals method. Two RssChannel objects will be equal if their
     * hashcodes (and thus their contents) are equal.
     * @param obj The other object
     * @return If the two objects are canonically equal
     */
    public boolean equals(Object obj){
        if(obj==null || !(obj instanceof RssChannelTextInput)){
            return false;
        }
        if(this==obj){
            return true;
        }
        return this.hashCode()==obj.hashCode();
    }

    /**
     * Public clone method to allow cloning.
     * @return This object's clone
     */
    public Object clone(){
        return this.clone();
    }

}