/*
 *  Copyright (c) 1999-2002 ChurchillObjects.com  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are
 *  met: Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer. Redistributions in
 *  binary form must reproduce the above copyright notice, this list of
 *  conditions and the following disclaimer in the documentation and/or other
 *  materials provided with the distribution. Neither the name of the copyright
 *  holder nor the names of its contributors may be used to endorse or promote
 *  products derived from this software without specific prior written
 *  permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
 *  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT, INCLUDING NEGLIGENCE OR OTHERWISE, ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 *  DAMAGE.
 *
 */

package churchillobjects.rss4j;

import java.io.Serializable;
import java.util.Enumeration;
import java.util.Vector;
import churchillobjects.rss4j.model.RssVersion;
/**
 * Encapsulates the data and some minor logic regarding a document in the RSS
 * object model. The document contains all of the elements inside of it,
 * beginning with the channel.
 */
public class RssDocument implements RssVersion, Cloneable, Serializable{

	/**
	 * Holds the channels for the RSS document.
	 */
	private Vector channels = new Vector();

	/**
	 * Holds the version for the RSS document.
	 */
	private String version;

	/**
	 * Holds the warnings that were generated by the parser when this document
	 * object was parsed from an RSS document.
	 */
	private Vector warnings = new Vector();

	/**
	 * Default constructor.
	 */
	public RssDocument(){
	}

	/**
	 * Overloaded constructor to set the version number.
	 * @param version
	 */
	public RssDocument(String version){
		setVersion(version);
	}

	/**
	 * Adds a channel to the document.
	 * @param channel
	 */
	public void addChannel(RssChannel channel){
		channels.addElement(channel);
	}

	/**
	 * Returns an enumeration of channels to the document.
	 * @return
	 */
	public Enumeration channels(){
		return channels.elements();
	}

	/**
	 * Sets the RSS version number of the document.
	 * @param s
	 */
	public void setVersion(String s){
		// fix some common argument alternatives
		if(s.equals("0.9") || s.equals(".90") || s.equals(".9")){
			s = "0.90";
		}
		if(s.equals(".91")){
			s = "0.91";
		}
		if(s.equals("1.00")){
			s = "1.0";
		}
		// now test argument for valid version
		if(VERSION_90.equals(s) || VERSION_91.equals(s) || VERSION_10.equals(s)){
			version = s;
		}
		else{
			throw new IllegalArgumentException("Invalid RSS version: '"+s+"' - must be one of 0.90|0.91|1.0");
		}
	}

	/**
	 * Returns the RSS version number of the document.
	 * @return
	 */
	public String getVersion(){
		return version;
	}

	/**
	 * Adds a warning to the document. This is intended to be used by the parser
	 * to communicate problems and inconsistencies with the RSS document,
	 * @param s
	 */
	public void addWarning(String s){
		if(!warnings.contains(s)){
			warnings.addElement(s);
		}
	}

	/**
	 * Returns an enumeration of the current warnings.
	 * @return
	 */
	public Enumeration warnings(){
		return warnings.elements();
	}

	/**
	 * Returns a string representation of this object.
	 * @return
	 */
	public String toString(){
		StringBuffer sb = new StringBuffer(1000);
		sb.append("RSS Document: version "+version+" Channels: "+channels.size()+"\n");
		Enumeration enumeration = channels.elements();
		while(enumeration.hasMoreElements()){
			sb.append(" "+enumeration.nextElement());
		}
		return sb.toString();
	}

	/**
	 * Computes the hashcode of this object based on the hashcodes of the
	 * objects it contains. Documents with identical contents but different versions
	 * will have different hashcodes.
	 * @return The object's hashcode
	 */
	public int hashCode() {
		int hc = 31;
		Enumeration e1 = channels.elements();
		while(e1.hasMoreElements()){
			Object obj = e1.nextElement();
			if(obj!=null){
				hc = 31 * hc + obj.hashCode();
			}
		}
		hc = 31 * hc + (version==null ? 0 : version.hashCode());
		return hc;
	}

	/**
	 * Canonical equals method. Two RssChannel objects will be equal if their
	 * hashcodes (and thus their contents) are equal.
	 * @param o The other object
	 * @return If the two objects are canonically equal
	 */
	public boolean equals( Object o )
    {
        if ( this == o )
        {
            return true;
        }
        if ( o == null || getClass() != o.getClass() )
        {
            return false;
        }

        final RssDocument that = (RssDocument) o;

        if ( channels != null ? !channels.equals( that.channels ) : that.channels != null )
        {
            return false;
        }
        if ( version != null ? !version.equals( that.version ) : that.version != null )
        {
            return false;
        }
        if ( warnings != null ? !warnings.equals( that.warnings ) : that.warnings != null )
        {
            return false;
        }

        return true;
    }

    /**
	  * Public clone method to allow cloning.
	  * @return This object's clone
	  */
	 public Object clone(){
		 return this.clone();
	 }
}