/*
 *  Copyright (c) 1999-2002 ChurchillObjects.com  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are
 *  met: Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer. Redistributions in
 *  binary form must reproduce the above copyright notice, this list of
 *  conditions and the following disclaimer in the documentation and/or other
 *  materials provided with the distribution. Neither the name of the copyright
 *  holder nor the names of its contributors may be used to endorse or promote
 *  products derived from this software without specific prior written
 *  permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
 *  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT, INCLUDING NEGLIGENCE OR OTHERWISE, ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 *  DAMAGE.
 *
 */

package churchillobjects.rss4j;

import java.io.Serializable;
import java.text.ParseException;
import java.util.Date;
import churchillobjects.rss4j.model.DateLanguageUtil;

/**
 * Encapsulates the attributes and some logic of the RDF dublin core extension
 * to RSS. Only classes that implement IUsesDublinCore can support RssDublinCore
 * (RssChannel, RssChannelImage, RssChannelItem, and RssChannelTextInput).
 */
public class RssDublinCore implements Cloneable, Serializable{

	/**
	 * The XML namespace of the dublin core.
	 */
	public static final String XMLNS = "xmlns:dc";

	/**
	 * The prefix for dublin core elements.
	 */
	public static final String PREFIX = "dc";

	/**
	 * The namespace uri for dublin core.
	 */
	public static final String XMLNS_VALUE = "http://purl.org/dc/elements/1.1/";

	/**
	 * The dublin core's title.
	 */
	private String title;

	/**
	 * Added dc:modified because rss4j is a dead project
	 */
	private Date modified;

	/**
	 * The dublin core's creator.
	 */
	private String creator;

	/**
	 * The dublin core's subject.
	 */
	private String subject;

	/**
	 * The dublin core's description.
	 */
	private String description;

	/**
	 * The dublin core's publisher.
	 */
	private String publisher;

	/**
	 * The dublin core's contributor.
	 */
	private String contributor;

	/**
	 * The dublin core's date as a java.util.Date.
	 */
	private Date date;

	/**
	 * The dublin core's date as a String in ISO 8601 format.
	 */
	private String date8601;

	/**
	 * The dublin core's type information.
	 */
	private String type;

	/**
	 * The dublin core's format.
	 */
	private String format;

	/**
	 * The dublin core's identifier.
	 */
	private String identifier;

	/**
	 * The dublin core's source.
	 */
	private String source;

	/**
	 * The dublin core's language.
	 */
	private String language;

	/**
	 * The dublin core's relation.
	 */
	private String relation;

	/**
	 * The dublin core's coverage information.
	 */
	private String coverage;

	/**
	 * The dublin core's rights information.
	 */
	private String rights;

	/**
	 * Sets the dublin core's title.
	 * @param s
	 */
	public void setTitle(String s){
		title = s;
	}

	/**
	 * Returns the dublin core's title.
	 * @return
	 */
	public String getTitle(){
		return title;
	}

	public void setModified(Date date)
	{
		this.modified = date;
	}

	public String getModified()
	{
        String result = null;
        if( this.modified != null )
            result = this.modified.toString();

        return result;
    }

	/**
	 * Sets the dublin core's creator.
	 * @param s
	 */
	public void setCreator(String s){
		creator = s;
	}

	/**
	 * Returns the dublin core's creator.
	 * @return
	 */
	public String getCreator(){
		return creator;
	}

	/**
	 * Sets the dublin core's subject.
	 * @param s
	 */
	public void setSubject(String s){
		subject = s;
	}

	/**
	 * Returns the dublin core's subject.
	 * @return
	 */
	public String getSubject(){
		return subject;
	}

	/**
	 * Sets the dublin core's description.
	 * @param s
	 */
	public void setDescription(String s){
		description = s;
	}

	/**
	 * Returns the dublin core's description.
	 * @return
	 */
	public String getDescription(){
		return description;
	}

	/**
	 * Sets the dublin core's publisher.
	 * @param s
	 */
	public void setPublisher(String s){
		publisher = s;
	}

	/**
	 * Returns the dublin core's publisher.
	 * @return
	 */
	public String getPublisher(){
		return publisher;
	}

	/**
	 * Sets the dublin core's contributor.
	 * @param s
	 */
	public void setContributor(String s){
		contributor = s;
	}

	/**
	 * Returns the dublin core's contributor.
	 * @return
	 */
	public String getContributor(){
		return contributor;
	}

	/**
	 * Sets the dublin core's date in ISO 8601 format.
	 * @param d
	 * @throws ParseException
	 */
	public void setDate(String d) throws ParseException{
		date = DateLanguageUtil.convert8601ToDate(d);
		date8601 = d;
	}

	/**
	 * Sets the dublin core's date as a java.util.Date.
	 * @param d
	 */
	public void setDate(Date d){
		date = d;
		date8601 = DateLanguageUtil.convertDateTo8601(d);
	}

	/**
	 * Returns the dublin core's date as a java.util.Date.
	 * @return
	 */
	public Date getDate(){
		return date;
	}

	/**
	 * Returns the dublin core's date in ISO 8601 format.
	 * @return
	 */
	public String getDateAs8601(){
		return date8601;
	}

	/**
	 * Sets the dublin core's type.
	 * @param s
	 */
	public void setType(String s){
		type = s;
	}

	/**
	 * Returns the dublin core's type.
	 * @return
	 */
	public String getType(){
		return type;
	}

	/**
	 * Sets the dublin core's format.
	 * @param s
	 */
	public void setFormat(String s){
		format = s;
	}

	/**
	 * Returns the dublin core's format.
	 * @return
	 */
	public String getFormat(){
		return format;
	}

	/**
	 * Sets the dublin core's identifier information.
	 * @param s
	 */
	public void setIdentifier(String s){
		identifier = s;
	}

	/**
	 * Returns the dublin core's identifier.
	 * @return
	 */
	public String getIdentifier(){
		return identifier;
	}

	/**
	 * Sets the dublin core's source.
	 * @param s
	 */
	public void setSource(String s){
		source = s;
	}

	/**
	 * Returns the dublin core's source.
	 * @return
	 */
	public String getSource(){
		return source;
	}

	/**
	 * Sets the dublin core's language.
	 * @param s
	 */
	public void setLanguage(String s){
		language = s;
	}

	/**
	 * Returns the dublin core's language.
	 * @return
	 */
	public String getLanguage(){
		return language;
	}

	/**
	 * Sets the dublin core's relation.
	 * @param s
	 */
	public void setRelation(String s){
		relation = s;
	}

	/**
	 * Returns the dublin core's relation.
	 * @return
	 */
	public String getRelation(){
		return relation;
	}

	/**
	 * Sets the dublin core's coverage information.
	 * @param s
	 */
	public void setCoverage(String s){
		coverage = s;
	}

	/**
	 * Returns the dublin core's coverage information.
	 * @return
	 */
	public String getCoverage(){
		return coverage;
	}

	/**
	 * Sets the dublin core's rights.
	 * @param s
	 */
	public void setRights(String s){
		rights = s;
	}

	/**
	 * Returns the dublin core's rights.
	 * @return
	 */
	public String getRights(){
		return rights;
	}

	/**
	 * Returns a string representation of this object.
	 * @return
	 */
	public String toString(){
		StringBuffer sb = new StringBuffer();
		if(title!=null){
			sb.append(" title='"+title+"'");
		}
		if(null != this.modified){
			sb.append(" modified='" + modified + "'");
		}
		if(title!=null){
			sb.append(" creator='"+creator+"'");
		}
		if(title!=null){
			sb.append(" subject='"+subject+"'");
		}
		if(title!=null){
			sb.append(" description='"+description+"'");
		}
		if(title!=null){
			sb.append(" publisher='"+publisher+"'");
		}
		if(title!=null){
			sb.append(" contributor='"+contributor+"'");
		}
		if(title!=null){
			sb.append(" date8601='"+date8601+"'");
		}
		if(title!=null){
			sb.append(" type='"+type+"'");
		}
		if(title!=null){
			sb.append(" format='"+format+"'");
		}
		if(title!=null){
			sb.append(" identifier='"+identifier+"'");
		}
		if(title!=null){
			sb.append(" source='"+source+"'");
		}
		if(title!=null){
			sb.append(" language='"+language+"'");
		}
		if(title!=null){
			sb.append(" relation='"+relation+"'");
		}
		if(title!=null){
			sb.append(" coverage='"+coverage+"'");
		}
		if(title!=null){
			sb.append(" rights='"+rights+"'");
		}
		return "["+sb.toString().trim()+"]";
	}

	/**
	 * Computes the hashcode of this object based on the hashcodes of the
	 * objects it contains. For semantic reasons, an object with all null
	 * values returns 0, the same as if it were null within another object.
	 * @return The object's hashcode
	 */
	public int hashCode(){
		int hc = (title==null ? 0 : title.hashCode());
		hc = 31 * hc + (creator==null ? 0 : creator.hashCode());
		hc = 31 * hc + (subject==null ? 0 : subject.hashCode());
		hc = 31 * hc + (description==null ? 0 : description.hashCode());
		hc = 31 * hc + (publisher==null ? 0 : publisher.hashCode());
		hc = 31 * hc + (contributor==null ? 0 : contributor.hashCode());
		hc = 31 * hc + (date==null ? 0 : date.hashCode());
		hc = 31 * hc + (type==null ? 0 : type.hashCode());
		hc = 31 * hc + (format==null ? 0 : format.hashCode());
		hc = 31 * hc + (identifier==null ? 0 : identifier.hashCode());
		hc = 31 * hc + (source==null ? 0 : source.hashCode());
		hc = 31 * hc + (language==null ? 0 : language.hashCode());
		hc = 31 * hc + (relation==null ? 0 : relation.hashCode());
		hc = 31 * hc + (coverage==null ? 0 : coverage.hashCode());
		hc = 31 * hc + (rights==null ? 0 : rights.hashCode());
		return hc;
	}

	/**
	 * Canonical equals method. Two RssChannel objects will be equal if their
	 * hashcodes (and thus their contents) are equal.
	 * @param obj The other object
	 * @return If the two objects are canonically equal
	 */
	public boolean equals(Object obj){
		if(obj==null || !(obj instanceof RssDublinCore)){
			return false;
		}
		if(this==obj){
			return true;
		}
		return this.hashCode()==obj.hashCode();
	}

	/**
	 * Public clone method to allow cloning.
	 * @return This object's clone
	 */
	public Object clone(){
		return this.clone();
	}


}