/*
 *  Copyright (c) 1999-2002 ChurchillObjects.com  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are
 *  met: Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer. Redistributions in
 *  binary form must reproduce the above copyright notice, this list of
 *  conditions and the following disclaimer in the documentation and/or other
 *  materials provided with the distribution. Neither the name of the copyright
 *  holder nor the names of its contributors may be used to endorse or promote
 *  products derived from this software without specific prior written
 *  permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
 *  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT, INCLUDING NEGLIGENCE OR OTHERWISE, ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 *  DAMAGE.
 *
 */

package churchillobjects.rss4j;

import java.io.Serializable;
import java.text.ParseException;
import java.util.Date;
import churchillobjects.rss4j.model.DateLanguageUtil;

/**
 * Encapsulates the attributes and some logic of the RDF syndication extension
 * to RSS. This includes the update period, frequency and update base. Only classes
 * that implement IUsesSyndication can support RssSyndication (RssChannel).
 */
public class RssSyndication implements Cloneable, Serializable{

	/**
	 * The XML namespace for syndication.
	 */
	public static final String XMLNS = "xmlns:sy";

	/**
	 * The prefix used by syndication.
	 */
	public static final String PREFIX = "sy";

	/**
	 * The namespace uri for syndication.
	 */
	public static final String XMLNS_VALUE = "http://purl.org/rss/1.0/modules/syndication/";

	/**
	 * Enumerated hourly value.
	 */
	public static final String HOURLY = "hourly";

	/**
	 * Enumerated daily value.
	 */
	public static final String DAILY = "daily";

	/**
	 * Enumerated weekly value.
	 */
	public static final String WEEKLY = "weekly";

	/**
	 * Enumerated monthly value.
	 */
	public static final String MONTHLY = "monthly";

	/**
	 * Enumerated yearly value.
	 */
	public static final String YEARLY = "yearly";

	/**
	 * The syndication's update period, default to "daily".
	 */
	private String updatePeriod = DAILY;

	/**
	 * The syndication's update frequency, default to 1.
	 */
	private int updateFrequency = 1;

	/**
	 * The syndication's update base as a string.
	 */
	private String updateBase;

	/**
	 * The syndication's update base as a java.util.Date.
	 */
	private Date updateBaseDate;

	/**
	 * Default constructor.
	 */
	public RssSyndication(){
	}

	/**
	 * Overloaded constructor for presetting the object's values.
	 * @param updatePeriod
	 * @param updateFrequency
	 * @param updateBase
	 * @throws ParseException
	 */
	public RssSyndication(String updatePeriod, int updateFrequency, String updateBase) throws ParseException{
		setUpdateBase(updateBase);
		setUpdatePeriod(updatePeriod);
		this.updateFrequency = updateFrequency;
	}

	/**
	 * Overloaded constructor for presetting the object's values.
	 * @param updatePeriod
	 * @param updateFrequency
	 * @param updateBase
	 */
	public RssSyndication(String updatePeriod, int updateFrequency, Date updateBase){
		setUpdateBase(updateBase);
		setUpdatePeriod(updatePeriod);
		this.updateFrequency = updateFrequency;
	}

	/**
	 * Sets the update base for the syndication as a String.
	 * @param s
	 * @throws ParseException
	 */
	public void setUpdateBase(String s) throws ParseException{
		updateBaseDate = DateLanguageUtil.convert8601ToDate(s);
		updateBase = s;
	}

	/**
	 * Sets the update base for the syndicaiton as a java.util.Date.
	 * @param date
	 */
	public void setUpdateBase(Date date){
		updateBaseDate = date;
		updateBase = DateLanguageUtil.convertDateTo8601(date);
	}

	/**
	 * Sets the update period of the syndication.
	 * @param s
	 */
	public void setUpdatePeriod(String s){
		if(!(HOURLY.equals(s) || DAILY.equals(s) || WEEKLY.equals(s)
			 || MONTHLY.equals(s) || YEARLY.equals(s))){
			throw new IllegalArgumentException("Invalid update period: '"+s+"' - must be one of {'"+HOURLY+"', '"+DAILY+"', '"+WEEKLY+"', '"+MONTHLY+"', '"+YEARLY+"'}");
		}
		this.updatePeriod = s;
	}

	/**
	 * Stes the update frequency of the syndication.
	 * @param i
	 */
	public void setUpdateFrequency(int i){
		if(i<1){
			throw new IllegalArgumentException("Update frequency must be positive integer: "+i);
		}
		updateFrequency = i;
	}

	/**
	 * Returns the update period of the syndication.
	 * @return
	 */
	public String getUpdatePeriod(){
		return updatePeriod;
	}

	/**
	 * Returns the update frequency of the syndication.
	 * @return
	 */
	public int getUpdateFrequency(){
		return updateFrequency;
	}

	/**
	 * Returns the update base of the syndication as a String.
	 * @return
	 */
	public String getUpdateBase(){
		return updateBase;
	}

	/**
	 * Returns the update base of the syndication as a java.util.Date.
	 * @return
	 */
	public Date getUpdateBaseAsDate(){
		return updateBaseDate;
	}

	/**
	 * Returns a string representation of this object.
	 * @return
	 */
	public String toString(){
		return "[updatePeriod='"+updatePeriod+
				"', updateFrequency='"+updateFrequency+
				"', updateBase='"+updateBase+
				"']";
	}

	/**
	 * Computes the hashcode of this object based on the hashcodes of the
	 * objects it contains. For semantic reasons, an object with all null
	 * values returns 0, the same as if it were null within another object.
	 * @return The object's hashcode
	 */
	public int hashCode(){
		int hc = (updatePeriod==null ? 0 : updatePeriod.hashCode());
		hc = 31 * hc + updateFrequency;
		hc = 31 * hc + (updateBase==null ? 0 : updateBase.hashCode());
		return hc;
	}

	/**
	 * Canonical equals method. Two RssChannel objects will be equal if their
	 * hashcodes (and thus their contents) are equal.
	 * @param obj The other object
	 * @return If the two objects are canonically equal
	 */
	public boolean equals(Object obj){
		if(obj==null || !(obj instanceof RssSyndication)){
			return false;
		}
		if(this==obj){
			return true;
		}
		return this.hashCode()==obj.hashCode();
	}

	/**
	 * Public clone method to allow cloning.
	 * @return This object's clone
	 */
	public Object clone(){
		return this.clone();
	}

}