/*
 *  Copyright (c) 1999-2002 ChurchillObjects.com  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are
 *  met: Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer. Redistributions in
 *  binary form must reproduce the above copyright notice, this list of
 *  conditions and the following disclaimer in the documentation and/or other
 *  materials provided with the distribution. Neither the name of the copyright
 *  holder nor the names of its contributors may be used to endorse or promote
 *  products derived from this software without specific prior written
 *  permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
 *  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT, INCLUDING NEGLIGENCE OR OTHERWISE, ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 *  DAMAGE.
 *
 */

package churchillobjects.rss4j.generator;

import java.util.Vector;
import java.util.Enumeration;

import org.w3c.dom.Element;
import churchillobjects.rss4j.*;

class RssGeneratorImpl100 extends RssGeneratorImpl091{

	RssGeneratorImpl100() throws RssGenerationException{
		super();
	}

	private static final String XMLNS_RDF = "http://www.w3.org/1999/02/22-rdf-syntax-ns#";
	protected static final String XMLNS = "http://purl.org/rss/1.0/";
	protected static final String ROOT_ELEMENT = "RDF";


	private boolean usesSyndication = false;
	private boolean usesDublinCore = false;
    private boolean usesJbnPatch = false;
	private Vector imageElements;
	private Vector itemElements;
	private Vector textInputElements;

	/**
	 * Keep all max lenghts at zero; limits are suggested but not
	 * enforced with RSS 1.0.
	 */
	protected void setMaxLengths(){
	}

	/**
	 * Starts a new RSS 1.0 document. The RssDocument object is needed to determine
	 * what extensions are necessary to include in the root element.
	 * @param data
	 */
	protected void createRssDocument(RssDocument data){
		checkExtensions(data);
		imageElements = new Vector();
		itemElements = new Vector();
		textInputElements = new Vector();
		doc = domImpl.createDocument(XMLNS, ROOT_ELEMENT, null);
		rootElement = doc.getDocumentElement();
		rootElement.setPrefix("rdf");
		rootElement.setAttribute("xmlns:rdf", XMLNS_RDF);
		rootElement.setAttribute("xmlns", XMLNS);
		if(usesDublinCore){
			rootElement.setAttribute(RssDublinCore.XMLNS, RssDublinCore.XMLNS_VALUE);
		}
		if(usesSyndication){
			rootElement.setAttribute(RssSyndication.XMLNS, RssSyndication.XMLNS_VALUE);
		}
        if(usesJbnPatch)
        {
            rootElement.setAttribute( RssJbnPatch.XMLNS, RssJbnPatch.XMLNS_VALUE );
        }
	}

	/**
	 * Creates a channel element, digging down from that point in the object hierarchy
	 * and invoking the other handlers.
	 * @param channel
	 * @throws RssGenerationException
	 */
	protected void handleChannel(RssChannel channel) throws RssGenerationException{
		Element channelElement = doc.createElement("channel");
		rootElement.appendChild(channelElement);
		String channelUri = channel.getChannelUri();
		if(channelUri==null){
			throw new RssGenerationException("The channel URI is required for RSS 1.0.");
		}
		validateUri(channelUri);
		channelElement.setAttribute("rdf:about", channelUri);
		handleChannelTitle(channel, channelElement);
		handleChannelDescription(channel, channelElement);
		handleChannelLink(channel, channelElement);
		handleImage(channel, channelElement);
		handleItems(channel, channelElement);
		handleTextInput(channel, channelElement);
		handleDublinCore(channel.getDublinCore(), channelElement);
		handleSyndication(channel, channelElement);
	}

	/**
	 * Completes the document by adding the images, items and text inputs at the
	 * end of the document.
	 * @throws RssGenerationException
	 */
	protected void finishDocument() throws RssGenerationException{
		handleDocumentImages();
		handleDocumentItems();
		handleDocumentTextInputs();
	}


	/**
	 * Adds the text input to the channel element, referencing it to the text
	 * input element that will be appended to the document.
	 * @param channel
	 * @param channelElement
	 */
	protected void handleTextInput(RssChannel channel, Element channelElement){
		RssChannelTextInput textInput = channel.getChannelTextInput();
		if(textInput!=null){
			textInputElements.addElement(textInput);
			Element textInputElement = doc.createElement("textinput");
			textInputElement.setAttribute("rdf:resource", textInput.getInputLink());
			channelElement.appendChild(textInputElement);
		}
	}

	/**
	 * Adds the text input(s) to the end of the document, referencing them
	 * to the ones included within the channel(s).
	 * @throws RssGenerationException
	 */
	protected void handleDocumentTextInputs() throws RssGenerationException{
		Enumeration enumeration = textInputElements.elements();
		while(enumeration.hasMoreElements()){
			RssChannelTextInput textInput = (RssChannelTextInput)enumeration.nextElement();
			if(textInput!=null){
				Element textInputElement = doc.createElement("textinput");
				textInputElement.setAttribute("rdf:about", textInput.getInputLink());
				handleTextInputTitle(textInput, textInputElement);
				handleTextInputDescription(textInput, textInputElement);
				handleTextInputName(textInput, textInputElement);
				handleTextInputLink(textInput, textInputElement);
				handleDublinCore(textInput.getDublinCore(), textInputElement);
				rootElement.appendChild(textInputElement);
			}
		}
	}

	/**
	 * Adds the items to the channel element, referencing them to the image elements
	 * that will be appended to the document.
	 * @param channel
	 * @param channelElement
	 * @throws RssGenerationException
	 */
	protected void handleItems(RssChannel channel, Element channelElement) throws RssGenerationException{
		if(channel.getItemCount()==0){
			throw new RssGenerationException("Each channel must have at least one item");
		}
		Enumeration enumeration = channel.items();
		Element itemsElement = doc.createElement("items");
		channelElement.appendChild(itemsElement);
		Element itemsSequenceElement = doc.createElement("rdf:Seq");
		itemsElement.appendChild(itemsSequenceElement);
		while(enumeration.hasMoreElements()){
			RssChannelItem item = (RssChannelItem)enumeration.nextElement();
			if(item!=null){
				itemElements.addElement(item);
				Element itemElement = doc.createElement("rdf:li");
				itemElement.setAttribute("rdf:resource", item.getItemLink());
				itemsSequenceElement.appendChild(itemElement);
			}
		}
	}


	protected void handleDocumentItems() throws RssGenerationException{
		Enumeration enumeration = itemElements.elements();
		while(enumeration.hasMoreElements()){
			RssChannelItem item = (RssChannelItem)enumeration.nextElement();
			if(item!=null){
				Element itemElement = doc.createElement("item");
				itemElement.setAttribute("rdf:about", item.getItemLink());
				handleItemTitle(item, itemElement);
				handleItemLink(item, itemElement);
				handleItemDescription(item, itemElement);
				handleDublinCore(item.getDublinCore(), itemElement);
                handleJbnPatch( item.getJbnPatch(), itemElement );
				rootElement.appendChild(itemElement);
			}
		}
	}


	/**
	 * Appends the image resource to the channel, adding it to the list
	 * of image elements for inclusion later.
	 * @param channel
	 * @param channelElement
	 */
	protected void handleImage(RssChannel channel, Element channelElement){
		RssChannelImage image = (RssChannelImage)channel.getChannelImage();
		imageElements.addElement(image);
		if(image!=null){
			Element imageElement = doc.createElement("image");
			imageElement.setAttribute("rdf:resource", image.getImageUrl());
			channelElement.appendChild(imageElement);
		}
	}

	/**
	 * Appends all of the images used by channels into the RDF document.
	 */
	protected void handleDocumentImages() throws RssGenerationException{
		Enumeration enumeration = imageElements.elements();
		while(enumeration.hasMoreElements()){
			RssChannelImage image = (RssChannelImage)enumeration.nextElement();
			if(image!=null){
				Element imageElement = doc.createElement("image");
				imageElement.setAttribute("rdf:about", image.getImageUrl());
				handleImageTitle(image, imageElement);
				handleImageUrl(image, imageElement);
				handleImageLink(image, imageElement);
				handleDublinCore(image.getDublinCore(), imageElement);
				rootElement.appendChild(imageElement);
			}
		}
	}

	protected void checkExtensions(RssDocument data){
		Enumeration channels = data.channels();
		while(channels.hasMoreElements()){
			RssChannel channel = (RssChannel)channels.nextElement();
			if(channel!=null){
				if(channel.getChannelSyndication()!=null){
					usesSyndication = true;
				}
				if(channel.getDublinCore()!=null){
					usesDublinCore = true;
				}
                Enumeration enumeration = channel.items();
                while( enumeration.hasMoreElements() )
                {
                    RssChannelItem item = (RssChannelItem)enumeration.nextElement();
                    if( item != null )
                    {
                        if( item.getDublinCore() != null )
                        {
                            usesDublinCore = true;
                        }
                        if( item.getJbnPatch() != null )
                        {
                            usesJbnPatch = true;
                        }
                    }
                }
			}
		}
	}

	protected void handleSyndication(RssChannel channel, Element channelElement) throws RssGenerationException{
		RssSyndication syn = channel.getChannelSyndication();
		if(syn!=null){
			if(syn.getUpdateBase()==null){
				throw new RssGenerationException("An update base is required for syndication.");
			}
			add(channelElement, "sy:updatePeriod", syn.getUpdatePeriod());
			add(channelElement, "sy:updateFrequency", ""+syn.getUpdateFrequency());
			add(channelElement, "sy:updateBase", syn.getUpdateBase());
		}
	}

	protected void handleDublinCore(RssDublinCore dc, Element channelElement){
		if(dc!=null){
			add(channelElement, "dc:title", dc.getTitle());
			add(channelElement, "dc:creator", dc.getCreator());
			add(channelElement, "dc:subject", dc.getSubject());
			add(channelElement, "dc:description", dc.getDescription());
			add(channelElement, "dc:publisher", dc.getPublisher());
			add(channelElement, "dc:contributor", dc.getContributor());
			add(channelElement, "dc:date", dc.getDateAs8601());
			add(channelElement, "dc:type", dc.getType());
			add(channelElement, "dc:format", dc.getFormat());
			add(channelElement, "dc:identifier", dc.getIdentifier());
			add(channelElement, "dc:source", dc.getSource());
			add(channelElement, "dc:language", dc.getLanguage());
			add(channelElement, "dc:relation", dc.getRelation());
			add(channelElement, "dc:coverage", dc.getCoverage());
			add(channelElement, "dc:rights", dc.getRights());
			add(channelElement, "dc:modified", dc.getModified());
		}
	}

    protected void handleJbnPatch( RssJbnPatch jbn, Element channelElement )
    {
        if( jbn != null )
        {
            add( channelElement, RssJbnPatch.PREFIX + ":" + RssJbnPatch.ATTR_CREATOR, jbn.getCreator() );
            add( channelElement, RssJbnPatch.PREFIX + ":" + RssJbnPatch.ATTR_DISTRIBUTION_STATUS, jbn.getDistributionStatus() );
            add( channelElement, RssJbnPatch.PREFIX + ":" + RssJbnPatch.ATTR_LICENSE_NAME, jbn.getLicenseName() );
            add( channelElement, RssJbnPatch.PREFIX + ":" + RssJbnPatch.ATTR_LICENSE_VERSION, jbn.getLicenseVersion() );
            add( channelElement, RssJbnPatch.PREFIX + ":" + RssJbnPatch.ATTR_TYPE, jbn.getType() );
            add( channelElement, RssJbnPatch.PREFIX + ":" + RssJbnPatch.ATTR_PRODUCTS,
                    RssJbnPatch.PREFIX + ":" + RssJbnPatch.ATTR_PRODUCT, jbn.getProducts() );
            add( channelElement, RssJbnPatch.PREFIX + ":" + RssJbnPatch.ATTR_JIRA, jbn.getJira() );
            add( channelElement, RssJbnPatch.PREFIX + ":" + RssJbnPatch.ATTR_CASE_ID, jbn.getCaseId() );
            add( channelElement, RssJbnPatch.PREFIX + ":" + RssJbnPatch.ATTR_MD5, jbn.getMd5() );
            add( channelElement, RssJbnPatch.PREFIX + ":" + RssJbnPatch.ATTR_SHA256, jbn.getSha256() );
            add( channelElement, RssJbnPatch.PREFIX + ":" + RssJbnPatch.ATTR_FILE_NAME, jbn.getFileName() );
            add( channelElement, RssJbnPatch.PREFIX + ":" + RssJbnPatch.ATTR_FILE_SIZE, jbn.getFileSize() );
            add( channelElement, RssJbnPatch.PREFIX + ":" + RssJbnPatch.ATTR_DOWNLOAD_URL, jbn.getDownloadUrl() );
            add( channelElement, RssJbnPatch.PREFIX + ":" + RssJbnPatch.ATTR_AUTOMATED_DOWNLOAD_URL, jbn.getAutomatedDownloadUrl() );            
            add( channelElement, RssJbnPatch.PREFIX + ":" + RssJbnPatch.ATTR_INSTRUCTION_VERSION, jbn.getInstructionCompatibilityVersion() );
            add( channelElement, RssJbnPatch.PREFIX + ":" + RssJbnPatch.ATTR_LONG_DESC, jbn.getLongDescription() );
            add( channelElement, RssJbnPatch.PREFIX + ":" + RssJbnPatch.ATTR_SHORT_DESC, jbn.getShortDescription() );
            add( channelElement, RssJbnPatch.PREFIX + ":" + RssJbnPatch.ATTR_MANUAL_INSTALL, jbn.getManualInstallation() );
            add( channelElement, RssJbnPatch.PREFIX + ":" + RssJbnPatch.ATTR_AUTOMATED_INSTALL, jbn.getAutomatedInstallation() );
            add( channelElement, RssJbnPatch.PREFIX + ":" + RssJbnPatch.ATTR_REQUIRES,
                    RssJbnPatch.PREFIX + ":" + RssJbnPatch.ATTR_PATCH, "rdf:about", jbn.getRequires() );
            add( channelElement, RssJbnPatch.PREFIX + ":" + RssJbnPatch.ATTR_IS_REPLACED_BY,
                    RssJbnPatch.PREFIX + ":" + RssJbnPatch.ATTR_PATCH, "rdf:about", jbn.getReplacedBy() );
            add( channelElement, RssJbnPatch.PREFIX + ":" + RssJbnPatch.ATTR_REPLACES,
                    RssJbnPatch.PREFIX + ":" + RssJbnPatch.ATTR_PATCH, "rdf:about", jbn.getReplaces() );
            add( channelElement, RssJbnPatch.PREFIX + ":" + RssJbnPatch.ATTR_COMPATIBLE_WITH,
                    RssJbnPatch.PREFIX + ":" + RssJbnPatch.ATTR_PATCH, "rdf:about", jbn.getCompatibleWith() );
            add( channelElement, RssJbnPatch.PREFIX + ":" + RssJbnPatch.ATTR_LAST_UPDATED, jbn.getLastUpdatedAsString() );

        }
    }

    //TODO: create a real jbn: namespace
    //TODO: create a repository for RSS4J  -- figure out how to contribute this code back
    //TODO: check in code                                                      
    //TODO: add unittests to make sure the RSS is being generated correctly
}