/*
 *  Copyright (c) 1999-2002 ChurchillObjects.com  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are
 *  met: Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer. Redistributions in
 *  binary form must reproduce the above copyright notice, this list of
 *  conditions and the following disclaimer in the documentation and/or other
 *  materials provided with the distribution. Neither the name of the copyright
 *  holder nor the names of its contributors may be used to endorse or promote
 *  products derived from this software without specific prior written
 *  permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
 *  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT, INCLUDING NEGLIGENCE OR OTHERWISE, ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 *  DAMAGE.
 *
 */

package churchillobjects.rss4j.model;

import java.text.ParseException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.StringTokenizer;
import java.util.TimeZone;

/**
 * Static functionality class that provides locale-type support, such as
 * identifying standard languages and ISO 8601 dates.
 */
public class DateLanguageUtil{
                                            //2005-05-12T20:01:Z
    public static final String DATE_FORMAT = "yyyy-MM-dd'T'HH:mm:'Z'";

    /**
	 * Prevent this class from being instantiated.
	 */
	private DateLanguageUtil() {
	}

	/**
	 * Hard-coded list of world languages and codes.
	 */
	private static String[][] languages = {
		{"Afrikaans", "af"},
		{"Albanian", "sq&nbsp;"},
		{"Basque", "eu"},
		{"Belarusian", "be"},
		{"Bulgarian", "bg"},
		{"Catalan", "ca"},
		{"Chinese (Simplified)", "zh-cn"},
		{"Chinese (Traditional)", "zh-tw"},
		{"Croatian", "hr"},
		{"Czech", "cs"},
		{"Danish", "da"},
		{"Dutch", "nl"},
		{"Dutch (Belgium)", "nl-be"},
		{"Dutch (Netherlands)", "nl-nl"},
		{"English", "en"},
		{"English (Australia)", "en-au"},
		{"English (Belize)", "en-bz"},
		{"English (Canada)", "en-ca"},
		{"English (Ireland)", "en-ie"},
		{"English (Jamaica)", "en-jm"},
		{"English (New Zealand)", "en-nz"},
		{"English (Phillipines)", "en-ph"},
		{"English (South Africa)", "en-za"},
		{"English (Trinidad)", "en-tt"},
		{"English (United Kingdom)", "en-gb"},
		{"English (United States)", "en-us"},
		{"English (Zimbabwe)", "en-zw"},
		{"Faeroese", "fo"},
		{"Finnish", "fi"},
		{"French", "fr"},
		{"French (Belgium)", "fr-be"},
		{"French (Canada)", "fr-ca"},
		{"French (France)", "fr-fr"},
		{"French (Luxembourg)", "fr-lu"},
		{"French (Monaco)", "fr-mc"},
		{"French (Switzerland)", "fr-ch"},
		{"Galician", "gl"},
		{"Gaelic", "gd"},
		{"German", "de"},
		{"German (Austria)", "de-at"},
		{"German (Germany)", "de-de"},
		{"German (Liechtenstein)", "de-li"},
		{"German (Luxembourg)", "de-lu"},
		{"German (Switzerland)", "de-ch"},
		{"Greek", "el"},
		{"Hungarian", "hu"},
		{"Icelandic", "is"},
		{"Indonesian", "in"},
		{"Irish", "ga"},
		{"Italian", "it"},
		{"Italian (Italy)", "it-it"},
		{"Italian (Switzerland)", "it-ch"},
		{"Japanese", "ja"},
		{"Korean", "ko"},
		{"Macedonian", "mk"},
		{"Norwegian", "no"},
		{"Polish", "pl"},
		{"Portuguese", "pt"},
		{"Portuguese (Brazil)", "pt-br"},
		{"Portuguese (Portugal)", "pt-pt"},
		{"Romanian", "ro"},
		{"Romanian (Moldova)", "ro-mo"},
		{"Romanian (Romania)", "ro-ro"},
		{"Russian", "ru"},
		{"Russian (Moldova)", "ru-mo"},
		{"Russian (Russia)", "ru-ru"},
		{"Serbian", "sr"},
		{"Slovak", "sk"},
		{"Slovenian", "sl"},
		{"Spanish", "es"},
		{"Spanish (Argentina)", "es-ar"},
		{"Spanish (Bolivia)", "es-bo"},
		{"Spanish (Chile)", "es-cl"},
		{"Spanish (Colombia)", "es-co"},
		{"Spanish (Costa Rica)", "es-cr"},
		{"Spanish (Dominican Republic)", "es-do"},
		{"Spanish (Ecuador)", "es-ec"},
		{"Spanish (El Salvador)", "es-sv"},
		{"Spanish (Guatemala)", "es-gt"},
		{"Spanish (Honduras)", "es-hn"},
		{"Spanish (Mexico)", "es-mx"},
		{"Spanish (Nicaragua)", "es-ni"},
		{"Spanish (Panama)", "es-pa"},
		{"Spanish (Paraguay)", "es-py"},
		{"Spanish (Peru)", "es-pe"},
		{"Spanish (Puerto Rico)", "es-pr"},
		{"Spanish (Spain)", "es-es"},
		{"Spanish (Uruguay)", "es-uy"},
		{"Spanish (Venezuela)es-ve"},
		{"Swedish", "sv"},
		{"Swedish (Finland)", "sv-fi"},
		{"Swedish (Sweden)", "sv-se"},
		{"Turkish", "tr"},
		{"Ukranian", "uk"}
	};

	/**
	* Identifies if a specified language code is valid (in the list).
	* @param s The language code
	* @return If the code is valid
	*/
	public static boolean isValidLanguage(String s){
		if(s==null || s.length()<2){
			return false;
		}
		for(int i=0;i<languages.length;i++){
			if(languages[i][1].equals(s)){
				return true;
			}
		}
		return false;
	}

	/**
	 * Converts a java.util.Date object to a string-represented ISO 8601 date.
	 * This conversion is limited by not using seconds (which are not of interest
	 * to RSS feeds) and also always setting the timezone to UTC (the 'Z' at the
	 * end).
	 * @param date
	 * @return
	 */
	public static String convertDateTo8601(Date date)
    {
        String result = null;
        SimpleDateFormat formatter = new SimpleDateFormat( DATE_FORMAT );
        //2005-12-14T16:44:Z

        if ( date != null )
        {
            result = formatter.format( date );
        }
        return result;
    }

    /**
	 * Creates a date from an ISO8601 date-string.
	 * @param string the string to be converted into a date
	 * @return the date
	 * @throws ParseException
	 */
    public static Date convert8601ToDate( String string ) throws ParseException
    {
        Date result = null;
        SimpleDateFormat formatter = new SimpleDateFormat( DATE_FORMAT );

        if ( string != null )
        {
            result =  formatter.parse( string );
        }

        return result;
    }

}