/*
 *  Copyright (c) 1999-2002 ChurchillObjects.com  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are
 *  met: Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer. Redistributions in
 *  binary form must reproduce the above copyright notice, this list of
 *  conditions and the following disclaimer in the documentation and/or other
 *  materials provided with the distribution. Neither the name of the copyright
 *  holder nor the names of its contributors may be used to endorse or promote
 *  products derived from this software without specific prior written
 *  permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
 *  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT, INCLUDING NEGLIGENCE OR OTHERWISE, ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 *  DAMAGE.
 *
 */

package churchillobjects.rss4j.parser;

import java.util.Vector;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import churchillobjects.rss4j.RssDocument;

/**
 * Class to parse an RSS document according to the specifications of the RSS
 * .91 standard. This object is created when the base parser already started
 * parsing the document (and knows what the version number is), so it is given
 * the document object and the known namespaces up to that point.
 */
class RssParserImpl091 extends RssParserImpl090{

	/**
	 * Indicates that the parser is inside the skipDays element, so it should look
	 * for days values to follow.
	 */
	protected boolean inSkipDays = false;

	/**
	 * Indicates that the parser is inside the skipHours element, so it should look
	 * for hours values to follow.
	 */
	protected boolean inSkipHours = false;

	/**
	 * Static version number string for this implementation.
	 */
	private static final String VERSION = "0.91";

	/**
	 * Constructor for this RSS verion's parser. Provided with an already
	 * set-up document and namespace.
	 * @param document
	 * @param namespaces
	 */
	RssParserImpl091(RssDocument document, Vector namespaces){
		super(document, namespaces);
	}

	/**
	 * SAX interface method, begins a new element. For RSS 0.91, all elements except
	 * for skipDays and skipHours are handled in the superclass, 0.90.
	 * @param uri
	 * @param name
	 * @param qName
	 * @param attrs
	 * @throws SAXException
	 */
	public void startElement(String uri, String name, String qName, Attributes attrs) throws SAXException{
		super.startElement(uri, name, qName, attrs);
		if("skipDays".equals(name)){
			inSkipDays = true;
		}
		if("skipHours".equals(name)){
			inSkipHours = true;
		}
	}

	/**
	 * SAX interface method, indicates that an element has completed. Set flags to
	 * indicate being outside an element, or obtain the characters of the element
	 * and add them to the document.
	 * @param uri
	 * @param name
	 * @param qName
	 * @throws SAXException
	 */
	public void endElement(String uri, String name, String qName) throws SAXException{
		try{
			// all RSS 0.90 elements apply to 0.91
			super.endElement(uri, name, qName);

			// now handle RSS 0.91-specific elements
			if("skipDays".equals(name)){
				inSkipDays = false;
			}
			if("skipHours".equals(name)){
				inSkipHours = false;
			}

			// all others are fields within others,
			// if no chars then no sense in continuing
			if(!hasChars()){
				return;
			}

			if("day".equals(name) && inSkipDays){
				currentChannel.addSkipDay(getChars());
			}
			if("hour".equals(name) && inSkipHours){
				handleHour();
			}

			// 0.91 attributes for channel
			if(inChannel && !inImage && !inItem && !inTextInput){
				handleChannelAttributes(name);
			}

			// 0.91 attributes for image
			if(inImage){
				handleImageAttributes(name);
			}

			if(inItem){
				if("description".equals(name)){
					currentItem.setItemDescription(getChars());
				}
			}

			// if this is the right version (not a subclass), then clear the characters
			if(VERSION.equals(document.getVersion())){
				resetChars();
			}
		}
		catch(RssParseException e){
			throw new SAXException(e);
		}
	}

	/**
	 * Handles parsing the hour value for channels.
	 * @throws RssParseException
	 */
	private void handleHour() throws RssParseException {
		try{
			int hour = Integer.parseInt(getChars());
			currentChannel.addSkipHour(hour);
		}
		catch(NumberFormatException e){
			throw new RssParseException(e);
		}
	}

	/**
	 * Handles channel attributes that may exist for RSS 0.91 channels.
	 * @param name
	 */
	private void handleChannelAttributes(String name){
		if("language".equals(name)){
			currentChannel.setChannelLanguage(getChars());
		}
		if("rating".equals(name)){
			currentChannel.setChannelPicsRating(getChars());
		}
		if("copyright".equals(name)){
			currentChannel.setChannelCopyright(getChars());
		}
		if("pubDate".equals(name)){
			currentChannel.setChannelPubDate(getChars());
		}
		if("lastBuildDate".equals(name)){
			currentChannel.setChannelLastBuildDate(getChars());
		}
		if("docs".equals(name)){
			currentChannel.setChannelDocs(getChars());
		}
		if("managingEditor".equals(name)){
			currentChannel.setChannelManagingEditor(getChars());
		}
		if("webMaster".equals(name)){
			currentChannel.setChannelWebmaster(getChars());
		}
	}

	/**
	 * handles image attributes that may exist for RSS 0.91 images.
	 * @param name
	 * @throws RssParseException
	 */
	private void handleImageAttributes(String name) throws RssParseException {
		if("width".equals(name)){
			try{
				int width = Integer.parseInt(getChars());
				currentImage.setImageWidth(width);
			}
			catch(NumberFormatException e){
				throw new RssParseException(e);
			}
		}
		if("height".equals(name)){
			try{
				int height = Integer.parseInt(getChars());
				currentImage.setImageHeight(height);
			}
			catch(NumberFormatException e){
				throw new RssParseException(e);
			}
		}
		if("description".equals(name)){
			currentImage.setImageDescription(getChars());
		}
	}

}